#include  <stdio.h>
#include  <io.h>
#include  <signal.h>
#include  <string.h>


#include "lanimacros.h"
#include "uart.h"

//define USE_RTSCTS and Buffersize in uart.h

static volatile unsigned char inbuffer[INPUTBUFFERSIZE];
static volatile unsigned char schreibzeiger=0;
static volatile unsigned char lesezeiger=0;
#ifdef USE_RTSCTS
static volatile unsigned char incounter=0;
#endif

static volatile unsigned char outbuffer[OUTPUTBUFFERSIZE];
static volatile unsigned char outschreibzeiger=0;
static volatile unsigned char outlesezeiger=0;
static volatile unsigned char outbusy=0;
static volatile unsigned char zeichenimbuffer=0;

// UARTINIT#############################
void inituart(void)
{
	setBit(P3DIR,BIT6|BIT7);
	clearBit(P3OUT,BIT6);
	setBit(P3OUT,BIT7);
	
   UCTL0  = CHAR;                        // 8-bit character
   UTCTL0 = SSEL0 + SSEL1;               // UCLK = SMCLK
   
   UBR00  = 0x45;                        // 8MHz 115200
   UBR10  = 0x00;                        // 8MHz 115200
   
   //UBR00  = 0x2a;                        // 8MHz 300
   //UBR10  = 0x68;                        // 8MHz 300
   
   //UBR00  = 0xa0;                        // 8MHz 19200
   //UBR10  = 0x01;                        // 8MHz 19200
   
   //UBR00  = 0x05;                        // 8MHz 2400
   //UBR10  = 0x0D;                        // 8MHz 2400
   
   UMCTL0 = 0x00;                        // 8MHz 115200 modulation
   ME1   |= UTXE0 + URXE0;               // Enable USART0 TXD/RXD
   IE1   |= URXIE0| UTXIE0;              // Enable USART0 RX interrupt
   P3SEL |= 0x30;                        // P3.4,5 = USART0 TXD/RXD
   P3DIR |= 0x10;                        // P3.4 output direction
   
#ifdef USE_RTSCTS	
   clearBit(RTSCTSDIR,CTS);				//CTS Input
   setBit(RTSCTSDIR,RTS);				//RTS Output
   clearBit(RTSCTSOUT,RTS);				//RTS LOW
#endif
}

unsigned char getchar(void)
{
	unsigned char buchstabe;
	
	if (lesezeiger==schreibzeiger)
	{
		return 0;
	}else 
	{
		buchstabe=inbuffer[lesezeiger];
		lesezeiger++;
		
		if(lesezeiger>=INPUTBUFFERSIZE)
			lesezeiger=0;
		
#ifdef USE_RTSCTS	
		incounter--;
		if (incounter < (INPUTBUFFERSIZE -20) )
			clearBit(RTSCTSOUT,RTS);				//Handshake
#endif

		return buchstabe;
	}
}


unsigned char readstring(unsigned char *returnbuffer)
{
	unsigned char x=0;
	unsigned char zeiger=lesezeiger;

	do
	{
		if(zeiger>=INPUTBUFFERSIZE)
			zeiger=0;
		
		
		if(zeiger==schreibzeiger)
			return 0;
			
		returnbuffer[x++]=inbuffer[zeiger];
			
	}while(inbuffer[zeiger++]!= '\n');
	
	if(zeiger>=INPUTBUFFERSIZE)
		zeiger=0;
	
	returnbuffer[x]= 0;
	lesezeiger=zeiger;
	
#ifdef USE_RTSCTS	
	incounter = incounter - x;
	if (incounter < (INPUTBUFFERSIZE -20) )
		clearBit(RTSCTSOUT,RTS);				//Handshake
#endif
	
	return x;
}
		
		
		
		
interrupt (UART0RX_VECTOR) wakeup usart0_rx(void)
{
	inbuffer[schreibzeiger]= RXBUF0;
	
	// test
	//if (inbuffer[schreibzeiger] == '\t')
	//	setBit(RTSCTSOUT,RTS);
	//
	
	schreibzeiger++;
	
	if(schreibzeiger>=INPUTBUFFERSIZE)
		schreibzeiger=0;
#ifdef USE_RTSCTS
	incounter++;
	if (incounter >= INPUTBUFFERSIZE -20)
		setBit(RTSCTSOUT,RTS);
#endif
}

// fuer printf, uprintf usw. in <stdio.h>
int putchar(int c)
{
	while(zeichenimbuffer>=OUTPUTBUFFERSIZE-1); 	//warten wenn kein Platz im Buffer ist
	
	bufferchar(c);									//Test aus
	
	return 0;
}

void bufferchar(char newchar)
{
#ifdef USE_RTSCTS	
	while(bitIsSet(RTSCTSIN,CTS));
#endif	
	clearBit(IE1,UTXIE0);						// TX Interrupt aus
	
	outbuffer[outschreibzeiger++] = newchar;
	zeichenimbuffer++;
	
	if(outschreibzeiger>=OUTPUTBUFFERSIZE)
		outschreibzeiger=0;
	
	
	
	if(outbusy==0)
	{
		
//test
#ifdef USE_RTSCTS	
	while(bitIsSet(RTSCTSIN,CTS));
#endif	
		
		
		TXBUF0 = outbuffer[outlesezeiger++]; 	// erstes Zeichen schreiben
		if(outlesezeiger >=OUTPUTBUFFERSIZE)
			outlesezeiger=0;
		outbusy=1;
		zeichenimbuffer--;
	}
	
	setBit(IE1,UTXIE0);							// TX Interrupt wieder an
}

interrupt (UART0TX_VECTOR) usart0_tx(void)
{
	if(outlesezeiger != outschreibzeiger)
	{
		
				
//test
#ifdef USE_RTSCTS	
	while(bitIsSet(RTSCTSIN,CTS));
#endif	
		
		
		TXBUF0 = outbuffer[outlesezeiger++];
		if(outlesezeiger >=OUTPUTBUFFERSIZE)
			outlesezeiger=0;
		zeichenimbuffer--;
	}else
	{
		outbusy=0;
	}
}




